# - Find the cppunit library.
# Find the native cppunit includes and library
# This module defines
#  CPPUNIT_INCLUDE_DIR, where to find jpeglib.h, etc.
#  CPPUNIT_LIBRARIES, the libraries needed to use CPPUNIT.
#  CPPUNIT_FOUND, If false, do not try to use CPPUNIT.
# also defined, but not for general use are
#  CPPUNIT_LIBRARY, where to find the CPPUNIT library.

IF(WIN32 AND MSVC)
  SET(CPPUNIT_HOME $ENV{CPPUNIT_HOME})
ELSE(WIN32 AND MSVC)
  FIND_PATH(CPPUNIT_INCLUDE_DIR cppunit/Priority.hh
    PATHS ${CMAKE_INCLUDE_PATH} /usr/local/include /usr/include
    PATH_SUFFIXES include
  )
ENDIF(WIN32 AND MSVC)

SET(CPPUNIT_NAMES ${CPPUNIT_NAMES} cppunitD cppunit)
FIND_LIBRARY(CPPUNIT_LIBRARY
  NAMES ${CPPUNIT_NAMES}
  PATHS ${CMAKE_INCLUDE_PATH} /usr/local/include /usr/include
  PATH_SUFFIXES lib
)

#message(STATUS "LIB: ${CPPUNIT_LIBRARY}, INC: ${CPPUNIT_INCLUDE_DIR}")

SET(CPPUNIT_INCLUDE_DIR "${CPPUNIT_HOME}/include")
SET(CPPUNIT_STATIC_LIBRARY_DIRS "${CPPUNIT_HOME}/include")
SET(CPPUNIT_STATIC_LIBRARIES "${CPPUNIT_HOME}/lib/cppunit.lib")

IF (CPPUNIT_LIBRARY AND CPPUNIT_INCLUDE_DIR)
    SET(CPPUNIT_LIBRARIES ${CPPUNIT_LIBRARY})
    SET(CPPUNIT_FOUND "YES")
ELSE (CPPUNIT_LIBRARY AND CPPUNIT_INCLUDE_DIR)
  SET(CPPUNIT_FOUND "NO")
ENDIF (CPPUNIT_LIBRARY AND CPPUNIT_INCLUDE_DIR)
    SET(CPPUNIT_FOUND "YES")


IF (CPPUNIT_FOUND)
   IF (NOT CPPUNIT_FIND_QUIETLY)
      MESSAGE(STATUS "Found CPPUNIT: ${CPPUNIT_LIBRARIES}")
   ENDIF (NOT CPPUNIT_FIND_QUIETLY)
ELSE (CPPUNIT_FOUND)
   IF (CPPUNIT_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find CPPUNIT library")
   ENDIF (CPPUNIT_FIND_REQUIRED)
ENDIF (CPPUNIT_FOUND)

# Deprecated declarations.
SET (NATIVE_CPPUNIT_INCLUDE_PATH ${CPPUNIT_INCLUDE_DIR} )
GET_FILENAME_COMPONENT (NATIVE_CPPUNIT_LIB_PATH ${CPPUNIT_LIBRARY} PATH)


message(STATUS "CPPUNIT Libpath: ${CPPUNIT_STATIC_LIBRARIES}")
message(STATUS "CPPUNIT INCpath: ${CPPUNIT_INCLUDE_DIR}")
message(STATUS "CPPUNIT: -I${CPPUNIT_STATIC_INCLUDE_DIRS} -L${CPPUNIT_STATIC_LIBRARY_DIRS} -l${CPPUNIT_STATIC_LIBRARIES}")

MARK_AS_ADVANCED(
  CPPUNIT_STATIC_LIBRARIES
  CPPUNIT_LIBRARY
  CPPUNIT_INCLUDE_DIR
)
